@@ -5,7 +5,7 @@ use ash::vk::{self, Handle};
 pub const SCALE_FACTOR: f32 = 3.;
 
 use crate::{
-    components::{panel::PanelInput, Panel},
+    components::{panel::PanelInput, Panel, UIPanel},
     resources::render_context::{create_push_constant, CLEAR_VALUES},
     texture::Texture,
     COLOR_FORMAT,
@@ -155,7 +155,7 @@ impl GuiContext {
             let (vertex_shader, vertex_stage) = create_shader(
                 include_bytes!("../../shaders/gui.vert.spv"),
                 vk::ShaderStageFlags::VERTEX,
-                &vulkan_context,
+                vulkan_context,
             )
             .expect("Unable to create vertex shader");
 
@@ -163,7 +163,7 @@ impl GuiContext {
             let (fragment_shader, fragment_stage) = create_shader(
                 include_bytes!("../../shaders/gui.frag.spv"),
                 vk::ShaderStageFlags::FRAGMENT,
-                &vulkan_context,
+                vulkan_context,
             )
             .expect("Unable to create fragment shader");
 
@@ -266,18 +266,19 @@ impl GuiContext {
         vulkan_context: &VulkanContext,
         render_context: &RenderContext,
         current_swapchain_image_index: usize,
+        ui_panel: &mut UIPanel,
         panel: &mut Panel,
     ) {
         let device = &vulkan_context.device;
         let frame = &render_context.frames[current_swapchain_image_index];
         let command_buffer = frame.command_buffer;
-        let framebuffer = panel.framebuffer;
-        let extent = panel.extent;
-        let (raw_input, panel_input) = handle_panel_input(panel);
+        let framebuffer = ui_panel.framebuffer;
+        let extent = panel.resolution;
+        let (raw_input, panel_input) = handle_panel_input(ui_panel, panel);
 
-        let text = panel.text.clone();
-        let mut updated_buttons = panel.buttons.clone();
-        let egui_context = &mut panel.egui_context;
+        let text = ui_panel.text.clone();
+        let mut updated_buttons = ui_panel.buttons.clone();
+        let egui_context = &mut ui_panel.egui_context;
 
         egui_context.begin_frame(raw_input);
         let inner_layout = egui::Layout::from_main_dir_and_cross_align(
@@ -286,7 +287,7 @@ impl GuiContext {
         );
 
         // GUI Layout
-        egui::CentralPanel::default().show(&egui_context, |ui| {
+        egui::CentralPanel::default().show(egui_context, |ui| {
             ui.with_layout(inner_layout, |ui| {
                 ui.heading(&text);
 
@@ -303,9 +304,11 @@ impl GuiContext {
                 }
 
                 if let Some(panel_input) = panel_input {
-                    let position = ui
-                        .painter()
-                        .round_pos_to_pixels(panel_input.cursor_location);
+                    let (x, y) = (
+                        panel_input.cursor_location.x / SCALE_FACTOR,
+                        panel_input.cursor_location.y / SCALE_FACTOR,
+                    );
+                    let position = ui.painter().round_pos_to_pixels((x, y).into());
                     let cursor_colour = if panel_input.trigger_value > 0.9 {
                         egui::Color32::LIGHT_BLUE
                     } else {
@@ -322,7 +325,7 @@ impl GuiContext {
         let texture = &egui_context.fonts().texture();
         if texture.version != self.font_texture_version {
             let _font_texture = update_font_texture(
-                &vulkan_context,
+                vulkan_context,
                 texture,
                 self.font_texture_descriptor_sets[0],
             );
@@ -330,9 +333,9 @@ impl GuiContext {
         }
 
         let clipped_meshes = egui_context.tessellate(shapes);
-        panel.buttons = updated_buttons;
-        let vertex_buffer = &panel.vertex_buffer;
-        let index_buffer = &panel.index_buffer;
+        ui_panel.buttons = updated_buttons;
+        let vertex_buffer = &ui_panel.vertex_buffer;
+        let index_buffer = &ui_panel.index_buffer;
 
         // begin render pass
         unsafe {
@@ -402,11 +405,11 @@ impl GuiContext {
         for egui::ClippedMesh(rect, mesh) in &clipped_meshes {
             // Update vertex buffer
             vertex_buffer
-                .update(&vulkan_context, &mesh.vertices)
+                .update(vulkan_context, &mesh.vertices)
                 .unwrap();
 
             // Update index buffer
-            index_buffer.update(&vulkan_context, &mesh.indices).unwrap();
+            index_buffer.update(vulkan_context, &mesh.indices).unwrap();
 
             // record draw commands
             unsafe {
@@ -460,11 +463,17 @@ impl GuiContext {
     }
 }
 
-fn handle_panel_input(panel: &mut Panel) -> (egui::RawInput, Option<PanelInput>) {
-    let mut raw_input = panel.raw_input.clone();
+fn handle_panel_input(
+    ui_panel: &mut UIPanel,
+    panel: &mut Panel,
+) -> (egui::RawInput, Option<PanelInput>) {
+    let mut raw_input = ui_panel.raw_input.clone();
     let panel_input = panel.input.take();
     if let Some(input) = &panel_input {
-        let pos = input.cursor_location;
+        let pos = egui::Pos2 {
+            x: input.cursor_location.x / SCALE_FACTOR,
+            y: input.cursor_location.y / SCALE_FACTOR,
+        };
         raw_input.events.push(egui::Event::PointerMoved(pos));
         if input.trigger_value >= 0. {
             raw_input.events.push(egui::Event::PointerButton {
@@ -478,7 +487,7 @@ fn handle_panel_input(panel: &mut Panel) -> (egui::RawInput, Option<PanelInput>)
         raw_input.events.push(egui::Event::PointerGone);
     }
 
-    return (raw_input, panel_input);
+    (raw_input, panel_input)
 }
 
 fn update_font_texture(
